//WARPMAC Framework
//v2.3 (SVN rev X)
/***************CHANGELOG*****************
+updated to use radios in slots 2 and 3
******************************************/
/*****************WARPMAC*****************
Description: This framework allows for custom
MAC implementations on WARP. For a description
of the functions, see:
http://warp.rice.edu/trac/wiki/WARPMAC
******************************************/
#include "xparameters.h"
#include "xemac.h"
#include <xstatus.h>
#include "xintc.h"
#include "xgpio.h"
#include <errno.h>
#include "xexception_l.h"

//Header files for WARP peripheral drivers
#include "ofdm_txrx_alamouti.h"
#include "ofdm_AGC_mimo.h"
#include "ofdm_timer.h"
#include "ofdm_pktDetector_mimo.h"
#include "EEPROM.h"
#include "radio_controller_basic.h"
#include "radio_controller_ext.h"
#include "radio_controller_adv.h"

//Header files for WARP libraries
#include "warpmac.h"

//This define enables per-packet printfs for debugging
// This *kills* performance & timing - use with caution
//#define DEBUG_PKT_PRINT

Maccontrol controlStruct;
XGpio Gpio;

////Globals for PHY Debugging////
int agctarget = -13;
int agcnoiseEst = -95;

unsigned int numTrainingSyms = 2;
unsigned int pktDly = 58;//251;
unsigned char globalReset = 0;

//numBaseRate must be zero in the current version of the Alamouti OFDM core
unsigned int numBaseRate = 0;

unsigned char mod_fullRate = 2;

//Define a bunch of cryptic constants for the PHY's phase tracking filter
#define INIT_A_KPVAL 0//0x7FFFFFFF//0xA6800
#define INIT_B_KIVAL 0x2500//xD000//x380000//0x7FFFFFFF//0xA6800
#define INIT_B_KPVAL 0xD0000//x2F000//x2620000//x7FFFFFFF//0x54afb0//0x1BC7D10//0x2C3C38//0xA6800
#define INIT_PN_KVAL 0x2700000//xFFFFFFFF
#define INIT_PN_KAlphaVAL 0x40000000
#define INIT_PN_KBetaVAL 0x40000000
#define INIT_PN_KGammaVAL 0x40000000
#define INIT_RXFFTOFSET 10
unsigned int A_KPval = INIT_A_KPVAL;
unsigned int B_KIval = INIT_B_KIVAL;
unsigned int B_KPval = INIT_B_KPVAL;
unsigned int PN_KVal = INIT_PN_KVAL;
unsigned int PN_KAlphaVal = INIT_PN_KAlphaVAL;
unsigned int PN_KBetaVal = INIT_PN_KBetaVAL;
unsigned int PN_KGammaVal = INIT_PN_KGammaVAL;
unsigned int RxFFT_Window_Offset = INIT_RXFFTOFSET;
/////////////////////////////////

///Global LED states////
unsigned int ledStates[LEDLEN] = {1,2,4,8,4,2};
unsigned int ledCounter = 0;
unsigned char ledStatesIndex = 0;
unsigned int ledStatesLow[2] = {1,2};
unsigned int ledCounterLow = 0;
unsigned char ledStatesIndexLow = 0;
unsigned int ledStatesHigh[2] = {4,8};
unsigned int ledCounterHigh = 0;
unsigned char ledStatesIndexHigh = 0;
////////////////////////

////Ethernet Parameters////
XEmac Emac;
XEmac *InstancePtr = &Emac;
static Xuint8 LocalAddress[XEM_MAC_ADDR_SIZE] = {0x01, 0x02, 0x03, 0x04, 0x05, 0x06}; //wired MAC address... just filler... not used
static volatile int RxMutex;  /* Shared var used for receive sequencing */
static volatile int TxMutex;  /* Shared var used for transmit sequencing */
static XStatus ErrorCode;     /* Shared var set when the error handler is called */
static XStatus RxStatus;      /* Shared var set when the recv handler is called */
static Xuint32 RxFrameLength; /* Shared var set when the recv handler is called */
////////////////////////////

//Interrupt Controller Driver Instance
static XIntc myIntc;

////User Callback Pointers////
//These the addresses pointed to by these variables are set by the
//user via the callback registration functions
void (*usr_upbutton) ();
void (*usr_leftbutton) ();
void (*usr_middlebutton) ();
void (*usr_rightbutton) ();
void (*usr_badPacketHandler) ();
void (*usr_goodPacketHandler) ();
void (*usr_timerHandler) ();
void (*usr_emacHandler) ();
//////////////////////////////

////Buffer Pointers////
//These allow DMA-enabled transfers to and from the user's memory space
Macframe* rxPacket;
Macframe* txPacket;

void nullHandler(void* param){
};


/*WARPMAC_PHYINTERRUPTCLEAR - Lowers the interrupt line from the transceriver*/
void warpmac_phyInterruptClear(){
	alamouti_mac2phy_PktAck();
	return;
}

/*EMACRX_INT_HANDLER - Handles the reception of packets from the emac core*/
static void emacRx_int_handler(void *Callback) {
	XEmac *EmacPtr = (XEmac *)Callback;
	Xuint32 length = MY_XEM_MAX_FRAME_SIZE;

	if(controlStruct.enableEthernetInt){
		RxStatus = XEmac_FifoRecv(EmacPtr, txPacket->data, &length);
		if(RxStatus != XST_SUCCESS){
			xil_printf("Error in Fifo Recv: %d\r\n", RxStatus);
			return;
		}
		RxMutex=1;
		usr_emacHandler(RxMutex,length);
	}
	else{
		RxMutex=0;
	}

	XIntc_Acknowledge(&myIntc,XPAR_OPB_INTC_0_ETHERNET_MAC_IP2INTC_IRPT_INTR);

}

/*EMACTX_INT_HANDLER - This handler is called when the emac is finished sending a packet*/
static void emacTx_int_handler(void *Callback){
	TxMutex=0;
	rxPacket->isNew=0;
}

/*EMACERR_INT_HANDLER - This handler is called when there is a problem with the emac*/
static void emacErr_int_handler(void *Callback, XStatus ErrCode){
	xil_printf("ERROR IN EMAC #%d\r\n",ErrCode);
}

/*WARPMAC_INCREMENTLED - Every BLINKEVERY (in warpmac.h) times this function is called,
the user LEDs will switch state. The end result is a visualization that looks like Kitt
from Knightrider.*/
void warpmac_incrementLED(){
	if(ledCounter==0){
		XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, ledStates[ledStatesIndex]);
		ledStatesIndex = (ledStatesIndex+1)%LEDLEN;
	}
	ledCounter = (ledCounter+1)%BLINKEVERY;
}

/*WARPMAC_INCREMENTLEDLOW - Much like WARPMAC_INCREMENTLED, but allows for independent
control of the bottom two LEDs*/
void warpmac_incrementLEDLow(){
	if(ledCounterLow==0){
		XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, (ledStatesLow[ledStatesIndexLow]|ledStatesHigh[ledStatesIndexHigh]));
		ledStatesIndexLow = (ledStatesIndexLow+1)%2;
	}
	ledCounterLow = (ledCounterLow+1)%BLINKEVERY;
}

/*WARPMAC_INCREMENTLEDHIGH - Much like WARPMAC_INCREMENTLED, but allows for independent
control of the top two LEDs*/
void warpmac_incrementLEDHigh(){
	if(ledCounterHigh==0){
		//xil_printf("%x\r\n", (ledStatesLow[ledStatesIndexLow]|ledStatesHigh[ledStatesIndexHigh]));
		XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, (ledStatesLow[ledStatesIndexLow]|ledStatesHigh[ledStatesIndexHigh]));
		ledStatesIndexHigh = (ledStatesIndexHigh+1)%2;

	}
	ledCounterHigh = (ledCounterHigh+1)%BLINKEVERY;
}


/*TIMER_A_INT_HANDLER - interrupt handler for the first timer peripheral
The timer interrupt handler gets called when a hardware timer set by the
user expires. The user's callback is called upon the completion of this event */
void timer_a_int_handler(void * baseaddr_p) {


	/* Clear the timer interrupt */
	ofdm_timer_clearInterrupt();
	int baseaddr = (int) baseaddr_p;
	controlStruct.timerEnabled = 0;
	//Call User Function
	usr_timerHandler(controlStruct.timerType);

}


/*rxPhyBad - interrupt handler for the receive PHY layer
This function is the ISR for the RX physical layer peripheral.
This function is only called if the packet fails CRC. The received
packet is passed to the user's callback.*/
void rxPhyBad(void *baseaddr_p) {

	unsigned char header[24];
	unsigned int numPayloadBytes;
	int length;


	alamouti_ofdmRx_copyHeader(header,24); //Copy just the header

	unsigned int b = (unsigned int)header[18];
	unsigned int a = (unsigned int)header[19];

	numPayloadBytes = a*256 + b;//(*(unsigned short *)(0xFF101000+18));

#ifdef DEBUG_PKT_PRINT
		xil_printf("\r\n******************\r\nBAD:");
		xil_printf(" (%x %x -> %d) ",a, b, numPayloadBytes);

		int i;
		for(i=0;i<24;i++)
		{
			xil_printf("%x.",header[i]);
		}

		xil_printf("\r\n");
#endif

		usr_badPacketHandler(rxPacket);
		alamouti_mac2phy_PktAck();
}


/*rxPhyGood - interrupt handler for the receive PHY layer
This function is the ISR for the RX physical layer peripheral.
This function is only called if the packet has no bit errors
(i.e., the packet passes CRCs in the PHY). The function copies
the contents from the PHY into a temporary buffer, which then
gets converted to the custom frame structure that the MAC
code understands. The received packet is passed to the
user's callback.*/
void rxPhyGood(void *baseaddr_p) {

	if(rxPacket->isNew==0){
		unsigned char header[24];
		unsigned int numPayloadBytes;
		int length;
		alamouti_ofdmRx_copyHeader(header,24); //Copy just the header
		rxPacket->pktType = header[13];
		rxPacket->destAddr[0] = header[3];
		rxPacket->destAddr[1] = header[2];
		rxPacket->destAddr[2] = header[1];
		rxPacket->destAddr[3] = header[0];
		rxPacket->destAddr[4] = header[7];
		rxPacket->destAddr[5] = header[6];
		rxPacket->srcAddr[0] = header[5];
		rxPacket->srcAddr[1] = header[4];
		rxPacket->srcAddr[2] = header[11];
		rxPacket->srcAddr[3] = header[10];
		rxPacket->srcAddr[4] = header[9];
		rxPacket->srcAddr[5] = header[8];
		rxPacket->pktRev = header[15];

		unsigned int b = (unsigned int)header[18];
		unsigned int a = (unsigned int)header[19];

		numPayloadBytes = a*256 + b;//(*(unsigned short *)(0xFF101000+18));

			if(controlStruct.sequenceVec[rxPacket->srcAddr[5]].sequenceNumber[1] == header[16]){
				controlStruct.packetDupe=1;	//if sequencing is enabled, this will be used to keep the packet from
											//being sent over ethernet
			}
			else{
				controlStruct.packetDupe=0;
				controlStruct.sequenceVec[rxPacket->srcAddr[5]].sequenceNumber[1] = header[16];
			}

#ifdef DEBUG_PKT_PRINT
			xil_printf("\r\n******************\r\nGOOD:");
			xil_printf(" (%d) ",numPayloadBytes);
			xil_printf(" (%d) ",ofdm_txrx_alamouti_ReadReg_Rx_packet_done(OFDM_BASEADDR));

			int i;
			for(i=0;i<24;i++)
			{
				xil_printf("%x.",header[i]);
			}

			xil_printf("\r\n");
#endif

			rxPacket->length = numPayloadBytes - 24 - 4; //4 bytes for checksum
			if(rxPacket->length < 0 || rxPacket->length>MY_XEM_MAX_FRAME_SIZE){
				rxPacket->length=0;
			}
			usr_goodPacketHandler(rxPacket);

	}


	alamouti_mac2phy_PktAck();

}

/*PB_INT_HANDLER - interrupt handler for the pushbuttons on the WARP board. This is a low-priority
interrupt because it is primarily used for debugging purposes. The various user callbacks are
executed, depending upon which button was depressed.*/
void pb_int_handler(void *baseaddr_p) {

	Xuint32 dsr;
	XStatus stat;
	char lock;

	dsr = XGpio_mGetDataReg(XPAR_PUSH_BUTTONS_4BIT_BASEADDR, 2);
	switch(dsr) {

		case 0x01:
			usr_middlebutton();
			break;

		case 0x02:
			usr_rightbutton();
			break;

		case 0x08:
			usr_upbutton();
			break;
		case 0x04:
			usr_leftbutton();
			break;

		default : {
		}

	}
	XGpio_InterruptClear(&Gpio, 0x3); // clear the interrupt
}

/*WARPMAC_CLEARTIMER - This function stops the ofdm_timer. Additionally it
will return the amount of time remaining before expiration.*/
int warpmac_clearTimer(){
	int time;
	time = ofdm_timer_timeLeft();
	ofdm_timer_stop();
	controlStruct.currBackoff=0;
	return time;
}

/*WARPMAC_SETTIMERVAL - This function sets the ofdm_timer to countdown for
a given number of clock cycles.*/
void warpmac_setTimerVal(Xuint32 clocks){
	ofdm_timer_setVal(clocks);
}

/*WARPMAC_STARTTIMER - This function starts the ofdm_timer in either
a CSMA or non-CSMA mode.*/
void warpmac_startTimer(unsigned char mode){
	ofdm_timer_setMode(mode);
	ofdm_timer_start();
}

/*RANDNUM- generates a uniform random value between [0,N-1]*/
int randNum(int N){
	return rand() >> (32-(N+6));

/*	int ans;
	int temp;
	temp = rand();
	ans = temp>>(32-(N+6));
	return ans;
*/
}

/*WARPMAC_CARRIERSENSE- returns a value corresponding to the instantaneous channel condition.*/
unsigned int warpmac_carrierSense(){
	return ofdm_pktDetector_mimo_ReadReg_pktDet_idleDifs(PKTDET_BASEADDR);
}

/*SETTIMER- function is responsible for enabling the hardware timer peripherals*/
int warpmac_setTimer(int type){
	unsigned int setTime;
	int myRandNum;
	int retval;
	switch(type) {
		case TIMEOUT:
			controlStruct.timerType=TIMEOUT;
			setTime = controlStruct.timeout*40;
			warpmac_setTimerVal(setTime);
			warpmac_startTimer(DISABLECSMA);
			return 0;
			break;

		case BACKOFF:
			controlStruct.timerType=BACKOFF;
			myRandNum = randNum(controlStruct.currBackoff);
			setTime = myRandNum*controlStruct.slotTime*40;
			if(controlStruct.currBackoff < controlStruct.maxBackoff){
				/*Note: Currently, I never decrement a backoff... I only reset it upon the next frame. Technically, CSMA-CA
				should decrement the window under certain circumstances (check the 802.11 standard for details)*/
				controlStruct.currBackoff = controlStruct.currBackoff + 1;
			}

				/* Set the reset value of the timer (seconds * 50,000,000) */
				if(setTime != 0){
					warpmac_setTimerVal(setTime);
					warpmac_startTimer(ENABLECSMA);
					return 0;
				}

				if(setTime == 0){
					usr_timerHandler(controlStruct.timerType);
				}
				break;

	}
}


/*WARPMAC_INIT- initializes the framework and all hardware peripherals.*/
int warpmac_init() {

	usr_upbutton = nullHandler;
	usr_leftbutton = nullHandler;
	usr_middlebutton = nullHandler;
	usr_rightbutton = nullHandler;
	usr_badPacketHandler = nullHandler;
	usr_goodPacketHandler = nullHandler;

	xil_printf("Initializing WARPMAC...\r\n");

	xil_printf("	Initializing Ethernet...");

	/////////////////////////////////Set up ACKMAC////////////////////////////////////////////////////

	xil_printf("	Initializing controlStruct...");

	controlStruct.enableSequencing = 0;
	controlStruct.packetDupe=0;
	controlStruct.mySequenceNum.sequenceNumber[1] = 0;
	controlStruct.maxReSend = 4;
	controlStruct.currBackoff = 0;
	controlStruct.timerEnabled = 0;
	controlStruct.maxBackoff = 5;
	xil_printf("complete!\r\n");

	int retval;

	xil_printf("	Initializing Interrupts...");
	// Initialize the LEDs
	XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, 0x0004);

	// Initialize the push buttons
	XGpio_Initialize(&Gpio, XPAR_PUSH_BUTTONS_4BIT_DEVICE_ID);

	XGpio_mSetDataDirection(XPAR_PUSH_BUTTONS_4BIT_BASEADDR, 1, 0x3);
	// Enable interrupts for the push buttons
	XGpio_InterruptEnable(&Gpio, XGPIO_IR_CH1_MASK);

	// Enable global interrupts. Both this and above line necessary to enable interrupts
	XGpio_InterruptGlobalEnable(&Gpio);


	XIntc_Initialize(&myIntc, XPAR_OPB_INTC_0_DEVICE_ID);


	XIntc_SetIntrSvcOption(XPAR_OPB_INTC_0_BASEADDR,XIN_SVC_ALL_ISRS_OPTION); //keeps system from "missing" interrupts...
																			  //services all queued interrupts



	XIntc_Connect(&myIntc, XPAR_OPB_INTC_0_PUSH_BUTTONS_4BIT_IP2INTC_IRPT_INTR,(XInterruptHandler)pb_int_handler, NULL);
	XIntc_Connect(&myIntc, XPAR_OPB_INTC_0_OFDM_TXRX_ALAMOUTI_OPBW_0_RX_INT_GOODPKT_INTR,(XInterruptHandler)rxPhyGood, NULL);
	XIntc_Connect(&myIntc, XPAR_OPB_INTC_0_OFDM_TXRX_ALAMOUTI_OPBW_0_RX_INT_BADPKT_INTR,(XInterruptHandler)rxPhyBad, NULL);
	XIntc_Connect(&myIntc, XPAR_OPB_INTC_0_OFDM_TIMER_OPBW_0_TIMEREXPIRE_INTR,(XInterruptHandler)timer_a_int_handler, NULL);
	XIntc_Connect(&myIntc, XPAR_OPB_INTC_0_ETHERNET_MAC_IP2INTC_IRPT_INTR,(XInterruptHandler) XEmac_IntrHandlerFifo, &Emac);



	XIntc_Start(&myIntc, XIN_REAL_MODE);
	XIntc_Enable(&myIntc, XPAR_OPB_INTC_0_PUSH_BUTTONS_4BIT_IP2INTC_IRPT_INTR);
	XIntc_Enable(&myIntc, XPAR_OPB_INTC_0_OFDM_TXRX_ALAMOUTI_OPBW_0_RX_INT_GOODPKT_INTR);
	XIntc_Enable(&myIntc, XPAR_OPB_INTC_0_OFDM_TXRX_ALAMOUTI_OPBW_0_RX_INT_BADPKT_INTR);
	XIntc_Enable(&myIntc, XPAR_OPB_INTC_0_OFDM_TIMER_OPBW_0_TIMEREXPIRE_INTR);
	XIntc_Enable(&myIntc, XPAR_OPB_INTC_0_ETHERNET_MAC_IP2INTC_IRPT_INTR);





	xil_printf("Setting up exceptions...\r\n");
	XExc_Init();
	XExc_RegisterHandler(XEXC_ID_NON_CRITICAL_INT,(XExceptionHandler)XIntc_InterruptHandler,&myIntc);
	XExc_mEnableExceptions(XEXC_NON_CRITICAL);



	xil_printf("complete!\r\n");
	/////////////////////////////


	/////////////////////////ETHERNET///////////////////////
	int i,l;
	XStatus Status;
	Xuint32 Options;
	Xuint16 DeviceId = XPAR_ETHERNET_MAC_DEVICE_ID;

	/*
     * Initialize the XEmac component. The device ID chosen should be
     * configured without DMA (see the xemac_g.c file)
     */
    Status = XEmac_Initialize(InstancePtr, DeviceId);
    if (Status != XST_SUCCESS)
    {
	 xil_printf("Failed to initialize EMAC\r\n");
        return XST_FAILURE;
    }

    /*
     * Run self-test on the device, which verifies basic sanity of the
     * device and the driver.
     */
    Status = XEmac_SelfTest(InstancePtr);
    if (Status != XST_SUCCESS)
    {
		xil_printf("Self Test Failed\r\n");
        return XST_FAILURE;
    }

    /*
     * Configure the device to be promiscuous
     */
	//
    //Options = XEmac_GetOptions(InstancePtr);
    Options = (XEM_INSERT_FCS_OPTION |
			   XEM_INSERT_PAD_OPTION |
			   XEM_UNICAST_OPTION |
			   XEM_BROADCAST_OPTION |
			   //XEM_POLLED_OPTION|
			   //XEM_OVWRT_ADDR_OPTION|
			   XEM_PROMISC_OPTION|
			   XEM_STRIP_PAD_FCS_OPTION);

    Status = XEmac_SetOptions(InstancePtr, Options);
    if (Status != XST_SUCCESS)
    {
	 xil_printf("Failed to set options of EMAC\r\n");
        return XST_FAILURE;
    }

    /*
     * Set the MAC Addrees
     */

    Status = XEmac_SetMacAddress(InstancePtr, LocalAddress); //Note: Even though we are setting a MAC
															 //Address for the node, it is in promiscuous
															 //mode
    if (Status != XST_SUCCESS)
    {
	 xil_printf("Failed to set MAC of EMAC\r\n");
        return XST_FAILURE;
    }

    /*
     * Start the device, which enables the transmitter and receiver
     */
	XEmac_SetFifoSendHandler(InstancePtr, InstancePtr,
                              (XEmac_FifoHandler)emacTx_int_handler);
    XEmac_SetFifoRecvHandler(InstancePtr, InstancePtr,
							 (XEmac_FifoHandler)emacRx_int_handler);
    XEmac_SetErrorHandler(InstancePtr, InstancePtr,
						  (XEmac_ErrorHandler)emacErr_int_handler);

    Status = XEmac_Start(InstancePtr);
    if (Status != XST_SUCCESS)
    {
	 xil_printf("Failed to start EMAC\r\n");
        return XST_FAILURE;
    }
	xil_printf("complete!\r\n");

	RxMutex=0;
	TxMutex=0;

	///////////////////////////////////////////////////////////////////////////////////////////////////

	////////////////////////////////////SET UP RADIO and PHY///////////////////////////////////////////

	xil_printf("	Initializing Radio Transmitter...");

	warpmac_CalibrateTxDCO();

	//Setup the radio controller
	WarpRadio_v1_Reset((unsigned int *)XPAR_RADIO_CONTROLLER_0_BASEADDR);
	WarpRadio_v1_TxLpfCornFreqCoarseAdj( 0x1, FIRST_RADIO | SECOND_RADIO);
//	WarpRadio_v1_TxLpfCornFreqCoarseAdj( 0x1, FIRST_RADIO | SECOND_RADIO);


	//Finally turn on the power amplifier
	WarpRadio_v1_24AmpEnable(FIRST_RADIO | SECOND_RADIO);
	WarpRadio_v1_SetTxTiming(FIRST_RADIO, 50, 0, 0, 0);
	WarpRadio_v1_SetTxTiming(SECOND_RADIO, 50, 255, 0, 0);
	WarpRadio_v1_SetTxGainTiming(FIRST_RADIO | SECOND_RADIO, 0x3F, 0xF, 2);
	WarpRadio_v1_BaseBandTxGain(0x0, FIRST_RADIO | SECOND_RADIO);

	xil_printf("complete!\r\n");
	//**** End Radio Transmitter Initialization ****

	//**** Begin Radio Receiver Initialization ****

	xil_printf("	Initializing Radio Receiver...");

	//Enable AGC control of receive gains
	WarpRadio_v1_SoftwareRxGainControl(0,FIRST_RADIO | SECOND_RADIO);
	/* Use these functions to override AGC in software */
	//WarpRadio_v1_SoftwareRxGainControl(1,FIRST_RADIO | SECOND_RADIO);
	//WarpRadio_v1_RxLNAGainControl(0x3,FIRST_RADIO | SECOND_RADIO);
	//WarpRadio_v1_RxVGAGainControl(0x10,FIRST_RADIO | SECOND_RADIO);

	WarpRadio_v1_RxLpfCornFreqCoarseAdj(0x0, FIRST_RADIO | SECOND_RADIO);
//	WarpRadio_v1_RxLpfCornFreqCoarseAdj(0x1, FIRST_RADIO | SECOND_RADIO);

	xil_printf("complete\r\n");
	//**** End Radio Receiver Initialization ****


	//**** Begin OFDM Transmitter Initialization ****

	xil_printf("	Initializing OFDM Transmitter...");

	#define TX_FFT_SCALING_STAGE1 1
	#define TX_FFT_SCALING_STAGE2 2
	#define TX_FFT_SCALING_STAGE3 3

	//Zero out the modulation RAM - only transmit pilots
	//bzero((unsigned int *)(XPAR_OFDM_TXRX_ALAMOUTI_OPBW_0_BASEADDR+ofdm_TxRx_alamouti_SMWO_TxModulation_OFFSET), (64/2)*3);

	alamouti_ofdmTx_init();
	alamouti_ofdmTx_setFFTScaling((unsigned int)(16*TX_FFT_SCALING_STAGE1 + 4*TX_FFT_SCALING_STAGE2 + 1*TX_FFT_SCALING_STAGE3));
	alamouti_ofdmTx_setPilot1Index((7 + ((64-7)<<16) ));
	alamouti_ofdmTx_setPilot1Value(0x8000);//c000 8000
	alamouti_ofdmTx_setPilot2Index((21 + ((64-21)<<16)));
	alamouti_ofdmTx_setPilot2Value(0x7fff);  //3fff 7fff
	alamouti_ofdmTx_setRandom(0);

	#ifdef DEBUG_PKT_PRINT
	xil_printf("numTxBytes: %d\r\n",ofdm_txrx_alamouti_ReadReg_Tx_NumPayloadBytes(XPAR_OFDM_TXRX_ALAMOUTI_OPBW_0_BASEADDR));
	#endif

	alamouti_ofdmTx_setNumSyms(numTrainingSyms + (numBaseRate*256));
	alamouti_ofdmTx_setPreambleScaling(0x42FF);//42FF);//0x40FF); //7fff
	alamouti_ofdmTx_setControlBits(
		(2000 << 16) | //Delay 2nd transmit by 1000 25MHz cycles
		(0 << 9) | //Transmit block A first
		(0 << 8) | //Disable retransmissions
		(2 << 4) | //Shift antenna B's preamble
		/*TX_RANDOM_MODE |*/
		/*TX_DISABLE_ANTB_PREAMBLE |*/
		TX_PILOT_SCRAMBLING
		);
	alamouti_ofdmTx_enable();

	//**** End OFDM Transmitter Initialization ****

	//**** Begin OFDM Receiver Initialization ****

	xil_printf("	Initializing OFDM Receiver...");
	#define FFT_SCALING_STAGE1 0	//1
	#define FFT_SCALING_STAGE2 1	//2
	#define FFT_SCALING_STAGE3 1	//1

		alamouti_ofdmRx_init();

		//Scaling options:
		//268439552 -> 2
		//201329664 -> 1.5
		//134219776 -> 1
		//33554944 -> 0.25
		//67109888 -> 0.5
		//100664832 -> 0.75
		//114086809 -> 0.85
//		unsigned int myScaling = 67109888;//201329664;//(2048 + 2048<<16);
		unsigned int myScaling = 134219776;

		alamouti_ofdmRx_setFFTScaling((unsigned int)(16*FFT_SCALING_STAGE1 + 4*FFT_SCALING_STAGE2 + 1*FFT_SCALING_STAGE3));
		alamouti_ofdmRx_setFFTWindowOffset(INIT_RXFFTOFSET); //8
		alamouti_ofdmRx_setRxScaling(myScaling);//134219776);//(unsigned int)((unsigned int)2048 + ((unsigned int)2048*(2^16))));//2048 + 2048<<16);//2048 + 2048*2^16);//134219776);//0x1e66 + (0x1e66<<16)); //.5 in Fix_16_11
		alamouti_ofdmRx_setCorrThresh(0x60);
		alamouti_ofdmRx_setEnergyThresh(0x0);
		alamouti_ofdmRx_setLongCorrParams(251-16  + (2500*65536)); //251 2500
		alamouti_ofdmRx_setPktDetDly(80); //+43, //52
		alamouti_ofdmRx_setNumOFDMSyms(numTrainingSyms + (numBaseRate*65536));

//		alamouti_ofdmRx_setByteNums( (unsigned int)(24 + (16<<16) + (17<<24) ));
		alamouti_ofdmRx_setByteNums( (unsigned int)(24 + (18<<16) + (19<<24) ));
//		alamouti_ofdmRx_setByteNums( (unsigned int)(24 + (22<<16) + (23<<24) ));

		alamouti_ofdmRx_setCFODly(16);

		//ofdm_TxRx_alamouti_WriteReg_Rx_PktDet_Delay(XPAR_OFDM_TXRX_ALAMOUTI_OPBW_0_BASEADDR, 43 + (12<<7) + (CFODlyval<<16));

		alamouti_ofdmRx_setCFO_A_KP(INIT_A_KPVAL);

		alamouti_ofdmRx_setCFO_B_KI(INIT_B_KIVAL);
		alamouti_ofdmRx_setCFO_B_KP(INIT_B_KPVAL);

		alamouti_ofdmRx_setPNTrack_K(INIT_PN_KVAL);
		alamouti_ofdmRx_setPNTrack_Kalpha(INIT_PN_KAlphaVAL);
		alamouti_ofdmRx_setPNTrack_Kbeta(INIT_PN_KBetaVAL);
		alamouti_ofdmRx_setPNTrack_Kgamma(INIT_PN_KGammaVAL);

		alamouti_ofdmRx_setOptions(
								//RESET_BER |
								REQ_LONG_CORR |
								DYNAMC_PKT_LENGTHS |
								REQ_TWO_LONG_CORR |
								REQ_SHORT_CORR |
								//BYPASS_CARR_REC |
								//CFO_USE_LTS |
								//CFO_USE_LONGCORR |
								USE_PILOT_ARCTAN |
								SWITCHING_DIVERSITY |
								QUAD_LONGCORR_EN |
								EXT_PKT_DETECT,
								INTR_BAD_PKTS | INTR_GOOD_PKTS
								);
		alamouti_ofdmRx_enable();

		//Configure Tx/Rx for QPSK
		//All three entries must match for the Alamouti model
		//warpmac_set_modulation(mod_fullRate, mod_fullRate, mod_fullRate);

		xil_printf("complete!\r\n");
		//**** End OFDM Receiver Initialization ****


		xil_printf("	Initializing AGC...");

		WarpRadio_v1_RxHpSoftControlDisable(FIRST_RADIO | SECOND_RADIO);
		//IMPORTANT!!!?
		WarpRadio_v1_RxHighPassCornerFreq(1, FIRST_RADIO | SECOND_RADIO);

		ofdm_AGC_Initialize(agcnoiseEst);
		ofdm_AGC_setNoiseEstimate(agcnoiseEst);
		ofdm_AGC_FiltSel(1);
		ofdm_AGC_SetDCO(1); //1
		ofdm_AGC_SetTarget(agctarget); //-7
		ofdm_AGC_Reset();
		xil_printf("complete!\r\n");
		////////////////////////////////////////////////////////////////
		////////////////SET UP RSSI PACKET DETECTOR/////////////////////
		xil_printf("	Initializing Packet Detection...");

		ofdm_pktDetector_mimo_WriteReg_csma_enableBusy(PKTDET_BASEADDR, 1);
		ofdm_pktDetector_mimo_WriteReg_csma_enableIdle(PKTDET_BASEADDR, 1);
		ofdm_pktDetector_mimo_WriteReg_pktDet_masterReset(PKTDET_BASEADDR, 1);
		ofdm_pktDetector_mimo_WriteReg_pktDet_detectionMask(PKTDET_BASEADDR, 3); //3 = radios slot 2 and 3
//		ofdm_pktDetector_mimo_WriteReg_pktDet_detectionMask(PKTDET_BASEADDR, 2); //2 = radio slot 3
//		ofdm_pktDetector_mimo_WriteReg_pktDet_detectionMask(PKTDET_BASEADDR, 1); //1 = radio slot 2
		ofdm_pktDetector_mimo_WriteReg_pktDet_detectionMode(PKTDET_BASEADDR, 1); //0 = AND, 1 = OR
		ofdm_pktDetector_mimo_WriteReg_pktDet_resetDuration(PKTDET_BASEADDR, 32);
		ofdm_pktDetector_mimo_WriteReg_pktDet_avgLen(PKTDET_BASEADDR, 16);
		ofdm_pktDetector_mimo_WriteReg_pktDet_avgThresh(PKTDET_BASEADDR, 7000);
		ofdm_pktDetector_mimo_WriteReg_csma_avgThresh(PKTDET_BASEADDR, 16000);
		ofdm_pktDetector_mimo_WriteReg_csma_difsPeriod(PKTDET_BASEADDR, 625);
		ofdm_pktDetector_mimo_WriteReg_pktDet_masterReset(PKTDET_BASEADDR, 0);
		ofdm_pktDetector_mimo_WriteReg_pktDet_reset(PKTDET_BASEADDR, 0);

		xil_printf("complete!\r\n");
		////////////////////////////////////////////////////////////////

		WarpRadio_v1_RxEnable(FIRST_RADIO | SECOND_RADIO);

return 0;

}


void warpmac_pollEthernet(int* callback()){
	xil_printf("Deprecated Function\r\n");
}

/*WARPMAC_SENDETHERNET- Pushes the given Macframe over ethernet*/
void warpmac_sendEthernet(Macframe* packet){

	if(!((controlStruct.enableSequencing) && (controlStruct.packetDupe)) && TxMutex==0){
		unsigned int length;
		length = (packet->length)-4;

		XEmac_FifoSend(InstancePtr, OFDM_TXRX_ALAMOUTI_RXPKTBUFFER_BASEADDR+24, length);

		TxMutex=1;
		rxPacket->isNew=1;

	}

	TxMutex = 0;
}


/*WARPMAC_SENDOFDM- performs the conversion from the packet structure to the byte array and
sends the packet over the air*/
int warpmac_sendOfdm(Macframe* packet){
    int i;
    unsigned int numPayloadBytes;
    unsigned int Npld;
	unsigned char header[24];

	ofdm_pktDetector_mimo_WriteReg_pktDet_reset(PKTDET_BASEADDR, 1);

// From multiplexing MIMO model
//	Npld = ((packet->length + 4 + (6*controlStruct.mod_fullRateA-1))/(6*controlStruct.mod_fullRateA)) * (sisoMode+1);


	Npld = 2 + ((packet->length + 4 + (6*mod_fullRate-1))/(6*mod_fullRate));
	Npld = Npld + (Npld%2);

	numPayloadBytes = 24 + packet->length + 4;

	alamouti_ofdmTx_setNumBytes(numPayloadBytes);
	alamouti_ofdmTx_setNumSyms(numTrainingSyms + (numBaseRate*256) + (Npld*65536));

	header[0]= (packet->destAddr[3]);
	header[1]= (packet->destAddr[2]);
	header[2]= (packet->destAddr[1]);
	header[3]= (packet->destAddr[0]);
	header[4]= (packet->srcAddr[1]);
	header[5]= (packet->srcAddr[0]);
	header[6]= (packet->destAddr[5]);
	header[7]= (packet->destAddr[4]);
	header[8]= (packet->srcAddr[5]);
	header[9]= (packet->srcAddr[4]);
	header[10]= (packet->srcAddr[3]);
	header[11]= (packet->srcAddr[2]);
	header[12]= 0;
	header[13]= packet->pktType;
	header[14]= mod_fullRate;
	header[15]= packet->pktRev;  //version/mode/revision
	header[16]= (controlStruct.mySequenceNum.sequenceNumber[1]); //0
	header[17]= 0;
	header[18]= (0xFF & numPayloadBytes); //less significant byte of total length
	header[19]= (0xFF & (numPayloadBytes >> 8)); //more significant byte of total length
	header[20]= 0;
	header[21]= 0;
	header[22]= 0;
	header[23]= 0;

	alamouti_ofdmTx_copyHeader(header,(size_t)24);
	alamouti_ofdmTx_copyPayload(packet->data, (size_t)(packet->length) );

#ifdef DEBUG_PKT_PRINT
	xil_printf("\r\n******************\r\nTX:");
	xil_printf(" (%d - %d) ",numPayloadBytes, Npld);

	for(i=0;i<24;i++)
	{
		xil_printf("%x.",header[i]);
	}

	xil_printf("\r\n");
#endif

	alamouti_mac2phy_pktTx(TXBLOCK, FIRST_RADIO | SECOND_RADIO);

	WarpRadio_v1_RxEnable(FIRST_RADIO | SECOND_RADIO);

	if(globalReset==0) ofdm_pktDetector_mimo_WriteReg_pktDet_reset(PKTDET_BASEADDR, 0);

	return 0;
}

/*WARPMAC_SETUPBUTTONHANDLER- attaches user callback to up button*/
void warpmac_setUpButtonHandler(void(*handler)()){
	usr_upbutton = handler;
}

/*WARPMAC_SETLEFTBUTTONHANDLER- attaches user callback to left button*/
void warpmac_setLeftButtonHandler(void(*handler)()){
	usr_leftbutton = handler;
}

/*WARPMAC_SETRIGHTBUTTONHANDLER- attaches user callback to right button*/
void warpmac_setRightButtonHandler(void(*handler)()){
	usr_rightbutton = handler;
}

/*WARPMAC_SETMIDDLEBUTTONHANDLER- attaches user callback to middle button*/
void warpmac_setMiddleButtonHandler(void(*handler)()){
	usr_middlebutton = handler;
}

/*WARPMAC_SETTIMERHANDLER- attaches user callback to timer*/
void warpmac_setTimerHandler(void(*handler)()){
   usr_timerHandler = handler;
}



/*WARPMAC_SETEMACHANDLER- attaches user callback to ethernet*/
void warpmac_setEmacHandler(void(*handler)()){
	usr_emacHandler = handler;
}

/*WARPMAC_SETGOODPACKETHANDLER- attaches user callback to good packet interrupt*/
void warpmac_setGoodPacketHandler(void(*handler)()){
	usr_goodPacketHandler = handler;
}

/*WARPMAC_SETBADPACKETHANDLER- attaches user callback to bad packet interrupt*/
void warpmac_setBadPacketHandler(void(*handler)()){
	usr_badPacketHandler = handler;
}


/*WARPMAC_INCREMENTRESEND- increments the resend counter for the current transmit.
Additionally, it will free the payload when the maximum number of retransmissions has
occured.*/
int warpmac_incrementResend(Macframe* packet){
	int status = 1;
	packet->currReSend = ( packet->currReSend+1)%((controlStruct.maxReSend)+1); //increment currReSend and wrap around max
	if( packet->currReSend == 0) { //if reSend wraps around, the packet will be dropped
		status = 0;
		//buffree(mbuftACK,packet->data);
		controlStruct.currBackoff=0;
	   	packet->isNew = 0;
	}
	return status;
}


/*WARPMAC_ALLOCATEPAYLOAD- readies a Macframe for transmission. This includes
allocating memory for the payload and setting Macframe parameters.*/
int warpmac_allocatePayload(Macframe* packet,unsigned int length){
	xil_printf("Deprecated Function\r\n");
}

/*WARPMAC_FREEPAYLOAD- Frees previously allocated memory. Proper use of this
function ensures no memory leaks due to unallocated memory.*/
int warpmac_freePayload(Macframe* packet){
	xil_printf("Deprecated Function\r\n");
}

/*WARPMAC_ADDRESSEDTOME- returns a value corresponding to whether the packet was destined
for this particular node.*/
int warpmac_addressedToMe(Macframe* packet){
	int i,sum;
	unsigned char tempBoolean;
	sum = 0;
	for(i=0;i<6;i++){
		sum = sum+(packet->destAddr[i] == controlStruct.selfAddr[i]);
				}
				tempBoolean = 0;
				if(sum==6) tempBoolean = 1;
				sum = 0;
				return tempBoolean;
}

/*WARPMAC_ADDRESSEDFROMTHEM- creturns a value corresponding to whether the packet was sourced
from a given address*/
int warpmac_addressedFromThem(Macframe* packet,unsigned char* addr){
	int i,sum;
	unsigned char tempBoolean;
	sum = 0;
	for(i=0;i<6;i++){
		sum = sum+(packet->srcAddr[i] == addr[i]);
				}
				tempBoolean = 0;
				if(sum==6) tempBoolean = 1;
				sum = 0;

				return tempBoolean;
}


/*WARPMAC_ADDRESSEDTOTHEM- creturns a value corresponding to whether the packet was sourced
from a given address*/
int warpmac_addressedToThem(Macframe* packet,unsigned char* addr){
	int i,sum;
	unsigned char tempBoolean;
	sum = 0;
	for(i=0;i<6;i++){
		sum = sum+(packet->destAddr[i] == addr[i]);
				}
				tempBoolean = 0;
				if(sum==6) tempBoolean = 1;
				sum = 0;

				return tempBoolean;
}




/*WARPMAC_SETMAXRESEND- sets the maximum number of resends. This value is used by
WARPMAC_INCREMENTRESEND.*/
void warpmac_setMaxResend(unsigned int c){
	controlStruct.maxReSend=c;
}

/*WARPMAC_SETMAXCW- sets the maximum contention window.*/
void warpmac_setMaxCW(unsigned int c){
	controlStruct.maxBackoff=c;
}

/*WARPMAC_SETCHANNEL- sets the frequency band to operate in.*/
void warpmac_setChannel(unsigned int c){
	unsigned int val;
	val = WarpRadio_v1_SetCenterFreq2GHz(c, FIRST_RADIO | SECOND_RADIO);

}

/*WARPMAC_SETMACADDR- sets wireless MAC address of this node.*/
int warpmac_setMacAddr(unsigned char* addr){
	controlStruct.selfAddr[5] = addr[5];
	controlStruct.selfAddr[4] = addr[4];
	controlStruct.selfAddr[3] = addr[3];
	controlStruct.selfAddr[2] = addr[2];
	controlStruct.selfAddr[1] = addr[1];
	controlStruct.selfAddr[0] = addr[0];
}

/*WARPMAC_SETTIMEOUT- sets the amount of time the node is willing to wait for an acknowledgement.*/
int warpmac_setTimeout(unsigned int time){
	controlStruct.timeout = time;
}

/*WARPMAC_SETSLOTTIME- sets the base backoff window.*/
int warpmac_setSlotTime(unsigned int time){
	controlStruct.slotTime = time;
}

/*WARPMAC_INTIMEOUT- returns a value corresponding to whether or not the node is in timeout.*/
int warpmac_inTimeout(){
	return controlStruct.timerType==TIMEOUT;
}

/*WARPMAC_GETMYID- reads the value from the user dip switches for use as node identification.*/
int warpmac_getMyId(){
	int ret;
	ret =  XGpio_mGetDataReg(XPAR_DIPSWS_4BIT_BASEADDR,1);
	XGpio_mSetDataReg(XPAR_LED_7SEGMENT_BASEADDR, 1, sevenSegmentMap(ret));
	//This function returns the value currently specified in the Dip Switch on the FPGA board.
	return ret;
}

/*WARPMAC_COPYPAYLOAD- copies the payload from the received to the provided Macframe.*/
int warpmac_copyPayload(Macframe* packet){

	if(!((controlStruct.enableSequencing) && (controlStruct.packetDupe))){

		alamouti_ofdmRx_copyPayload(packet->data, (size_t)(16 + packet->length));

	}
}
void warpmac_CalibrateTxDCO () {
	unsigned char i;
	int eepromStatus = 0;
	short calReadback = 0;
	signed short best_I, best_Q;
	unsigned int thisRadio;

	unsigned int selectedRadios[4] = {0, RADIO2_ADDR, RADIO3_ADDR, 0};

	for(i=0; i<4; i++)
	{
		thisRadio = selectedRadios[i];
		//Only read values for enabled radios
		if(thisRadio > 0)
		{
			//Select the EEPROM on the current radio board
			eepromStatus = WarpEEPROM_EEPROMSelect((unsigned int*)XPAR_EEPROM_CONTROLLER_BASEADDR, i+1);

			//Initialize the EEPROM controller
			eepromStatus = WarpEEPROM_Initialize((unsigned int*)XPAR_EEPROM_CONTROLLER_BASEADDR);

			//Read the Tx DCO values
			calReadback = WarpEEPROM_ReadRadioCal((unsigned int*)XPAR_EEPROM_CONTROLLER_BASEADDR, 2, 0);

			//Scale the stored values
			best_I = (signed short)(((signed char)(calReadback & 0xFF))<<1);
			best_Q = (signed short)(((signed char)((calReadback>>8) & 0xFF))<<1);

			if(best_I == -1 & best_Q == 0) { while(1) {} }

			//Optional debug output; useful to check whether the values are valid
			xil_printf("\r\nRadio #%d TxDCO - I: %d Q: %d\r\n", i+1, best_I, best_Q);

			//Finally, write the Tx DCO values to the DAC
			WarpRadio_v1_DACOffsetAdj(ICHAN, best_I, thisRadio);
			WarpRadio_v1_DACOffsetAdj(QCHAN, best_Q, thisRadio);
			xil_printf("Finished setting radio %d\r\n", i+1);
		}
	}

	return;
}

/*WARPMAC_ENABLESISOMODE- sets the MIMO OFDM core into a SISO mode for legacy reasons.*/
void warpmac_enableSisoMode(){
	//NOT SUPPORTED IN ALAMOUTI MODEL
	return;
}

/*WARPMAC_ENABLEMIMOMODE- sets the MIMO OFDM core into the MIMO mode.*/
void warpmac_enableMimoMode(){
	//NOT SUPPORTED IN ALAMOUTI MODEL
	return;
}

/*WARPMAC_SETNUMTRAININGSYMS- sets the number of training symbols
used by the OFDM core*/
void warpmac_setNumTrainingSyms(unsigned int c){
	numTrainingSyms = c; //4
	//Only update Rx here; Tx gets new value on every pkt transmit
	alamouti_ofdmRx_setNumOFDMSyms(numTrainingSyms + (numBaseRate*65536));
}


unsigned char sevenSegmentMap(unsigned char x)
{
	switch(x)
	{
		case(0x0) : return 0x007E;
		case(0x1) : return 0x0030;
		case(0x2) : return 0x006D;
		case(0x3) : return 0x0079;
		case(0x4) : return 0x0033;
		case(0x5) : return 0x005B;
		case(0x6) : return 0x005F;
		case(0x7) : return 0x0070;
		case(0x8) : return 0x007F;
		case(0x9) : return 0x007B;

		case(0xA) : return 0x0077;
		case(0xB) : return 0x007F;
		case(0xC) : return 0x004E;
		case(0xD) : return 0x007E;
		case(0xE) : return 0x004F;
		case(0xF) : return 0x0047;
		default : return 0x0000;
	}
}

void warpmac_enableCSMA(){
	//10000 is large, but will be exceeded for nearby transmissions
	ofdm_pktDetector_mimo_WriteReg_csma_avgThresh(PKTDET_BASEADDR, 7000);
	return;
}

void warpmac_disableCSMA(){
	//16000 is big enough to never be exceeded
	ofdm_pktDetector_mimo_WriteReg_csma_avgThresh(PKTDET_BASEADDR, 16000);
	return;
}

/*WARPMAC_ENABLESEQUENCING- prevents duplicate packets from appearing over the wire*/
void warpmac_enableSequencing(){
	controlStruct.enableSequencing=1;
}

/*WARPMAC_DISABLESEQUENCING- allows duplicate packets from appearing over the wire*/
void warpmac_disableSequencing(){
	controlStruct.enableSequencing=0;
}


void warpmac_setPktDlyPlus(){
	pktDly++;
	xil_printf("+PktDly = %d\r\n",pktDly);
	alamouti_ofdmRx_setPktDetDly(pktDly);
}
void warpmac_setPktDlyMinus(){
	pktDly--;
	xil_printf("-PktDly = %d\r\n",pktDly);
	alamouti_ofdmRx_setPktDetDly(pktDly);
}

/****************************/
/* Phase noise tracking */
/****************************/
void print_PN_constants ()
{
	xil_printf("K: %x\t A: %x\t B: %x\t G: %x\r\n", PN_KVal, PN_KAlphaVal, PN_KBetaVal, PN_KGammaVal);
}

void warpmac_set_PN_KPlus(unsigned int increment){
	PN_KVal=PN_KVal+increment;
	alamouti_ofdmRx_setPNTrack_K(PN_KVal);
	print_PN_constants();
}

void warpmac_set_PN_KMinus(unsigned int decrement){
	PN_KVal=PN_KVal-decrement;
	alamouti_ofdmRx_setPNTrack_K(PN_KVal);
	print_PN_constants();
}

void warpmac_set_PN_KAlphaPlus(unsigned int increment){
	PN_KAlphaVal=PN_KAlphaVal+increment;
	alamouti_ofdmRx_setPNTrack_Kalpha(PN_KAlphaVal);
	print_PN_constants();
}

void warpmac_set_PN_KAlphaMinus(unsigned int decrement){
	PN_KAlphaVal=PN_KAlphaVal-decrement;
	alamouti_ofdmRx_setPNTrack_Kalpha(PN_KAlphaVal);
	print_PN_constants();
}

void warpmac_set_PN_KBetaPlus(unsigned int increment){
	PN_KBetaVal=PN_KBetaVal+increment;
	alamouti_ofdmRx_setPNTrack_Kbeta(PN_KBetaVal);
	print_PN_constants();
}

void warpmac_set_PN_KBetaMinus(unsigned int decrement){
	PN_KBetaVal=PN_KBetaVal-decrement;
	alamouti_ofdmRx_setPNTrack_Kbeta(PN_KBetaVal);
	print_PN_constants();
}

void warpmac_set_PN_KGammaPlus(unsigned int increment){
	PN_KGammaVal=PN_KGammaVal+increment;
	alamouti_ofdmRx_setPNTrack_Kgamma(PN_KGammaVal);
	print_PN_constants();
}

void warpmac_set_PN_KGammaMinus(unsigned int decrement){
	PN_KGammaVal=PN_KGammaVal-decrement;
	alamouti_ofdmRx_setPNTrack_Kgamma(PN_KGammaVal);
	print_PN_constants();
}

/****************************/

/****************************/
/* CFO Constants */
/****************************/

void print_CFO_constants ()
{
	xil_printf("AK: %x\tBP: %x\tBI: %x\r\n", A_KPval, B_KPval, B_KIval);
}

/*
void warpmac_set_A_KPPlus(unsigned int increment){
	A_KPval=A_KPval+increment;
	alamouti_ofdmRx_setCFO_A_KP(A_KPval);
	print_CFO_constants();
}

void warpmac_set_A_KPMinus(unsigned int decrement){
	A_KPval=A_KPval-decrement;
	alamouti_ofdmRx_setCFO_A_KP(A_KPval);
	print_CFO_constants();
}
*/

void warpmac_set_B_KPPlus(unsigned int increment){
	B_KPval=B_KPval+increment;
	alamouti_ofdmRx_setCFO_B_KP(B_KPval);
	print_CFO_constants();
}

void warpmac_set_B_KPMinus(unsigned int decrement){
	B_KPval=B_KPval-decrement;
	alamouti_ofdmRx_setCFO_B_KP(B_KPval);
	print_CFO_constants();
}

void warpmac_set_B_KIPlus(unsigned int increment){
	B_KIval=B_KIval+increment;
	alamouti_ofdmRx_setCFO_B_KI(B_KIval);
	print_CFO_constants();
}

void warpmac_set_B_KIMinus(unsigned int decrement){
	B_KIval=B_KIval-decrement;
	alamouti_ofdmRx_setCFO_B_KI(B_KIval);
	print_CFO_constants();
}

/****************************/

void warpmac_set_FFTOffset_Plus(){
	RxFFT_Window_Offset++;
	alamouti_ofdmRx_setFFTWindowOffset(RxFFT_Window_Offset);
	xil_printf("Rx FFT Offset: %d\r\n", RxFFT_Window_Offset);
}

void warpmac_set_FFTOffset_Minus(){
	RxFFT_Window_Offset--;
	alamouti_ofdmRx_setFFTWindowOffset(RxFFT_Window_Offset);
	xil_printf("Rx FFT Offset: %d\r\n", RxFFT_Window_Offset);
}

void warpmac_setNoiseTargetPlus(){
	agcnoiseEst++;
	xil_printf("+Noise Estimate = %d\r\n",agcnoiseEst);
	ofdm_AGC_setNoiseEstimate(agcnoiseEst);
}

void warpmac_setNoiseTargetMinus(){
	agcnoiseEst--;
	xil_printf("-Noise Estimate = %d\r\n",agcnoiseEst);
	ofdm_AGC_setNoiseEstimate(agcnoiseEst);
}

void warpmac_setTargetPlus(){
	agctarget++;
	xil_printf("+agctarget = %d\r\n",agctarget);
	XGpio_mSetDataReg(XPAR_LED_7SEGMENT_BASEADDR, 1, sevenSegmentMap(-agctarget%10));
	XGpio_mSetDataReg(XPAR_LED_7SEGMENT_1_BASEADDR, 1, sevenSegmentMap(-agctarget/10));
	ofdm_AGC_SetTarget(agctarget);
}

void warpmac_setTargetMinus(){
	agctarget=agctarget-1;
	xil_printf("-agctarget = %d\r\n",agctarget);
	XGpio_mSetDataReg(XPAR_LED_7SEGMENT_BASEADDR, 1, sevenSegmentMap(-agctarget%10));
	XGpio_mSetDataReg(XPAR_LED_7SEGMENT_1_BASEADDR, 1, sevenSegmentMap(-agctarget/10));
	ofdm_AGC_SetTarget(agctarget);
}

void warpmac_leftHex(unsigned char x){
	XGpio_mSetDataReg(XPAR_LED_7SEGMENT_1_BASEADDR, 1, sevenSegmentMap(x));
}
void warpmac_rightHex(unsigned char x){
	XGpio_mSetDataReg(XPAR_LED_7SEGMENT_BASEADDR, 1, sevenSegmentMap(x));
}

void warpmac_disableDetection(){
	globalReset = 1;
	ofdm_pktDetector_mimo_WriteReg_pktDet_reset(PKTDET_BASEADDR, 1);
}

void warpmac_enableDetection(){
	globalReset = 0;
	ofdm_pktDetector_mimo_WriteReg_pktDet_reset(PKTDET_BASEADDR, 0);
}

void warpmac_setRxBuffer(Macframe* packet){
	rxPacket=packet;
}

void warpmac_setTxBuffer(Macframe* packet){
	txPacket=packet;
}

void warpmac_printDestAddr(Macframe* packet){
	int i,sum;
	unsigned char tempBoolean;
	sum = 0;
	for(i=0;i<6;i++){
		xil_printf("%x\r\n",packet->destAddr[i]);
				}

}


void warpmac_set_modulation(unsigned char baseRate, unsigned char antAFullRate, unsigned char antBFullRate)
{
	unsigned char modIndex;
	unsigned int baseRateMask, antAMask, antBMask;

	//Define the standard subcarrier mapping - 48 used for data (0xF here), 4 pilots & 12 unused (0x0 here)
	unsigned int modMasks[24] = {0x0FFFFFF0, 0xFFFFFFFF, 0xFF0FFFFF, 0x00000FFF, 0xFF000000, 0xFFFF0FFF, 0xFFFFFFFF, 0xFFFFFF0F, 0x0FFFFFF0, 0xFFFFFFFF, 0xFF0FFFFF, 0x00000FFF, 0xFF000000, 0xFFFF0FFF, 0xFFFFFFFF, 0xFFFFFF0F, 0x0FFFFFF0, 0xFFFFFFFF, 0xFF0FFFFF, 0x00000FFF, 0xFF000000, 0xFFFF0FFF, 0xFFFFFFFF, 0xFFFFFF0F};

	switch(baseRate)
	{
		case 1:
			baseRateMask = MODMASK_BPSK;
			break;
		case 2:
			baseRateMask = MODMASK_QPSK;
			break;
		case 4:
			baseRateMask = MODMASK_16QAM;
			break;
		case 6:
			baseRateMask = MODMASK_64QAM;
			break;
		default:
			baseRateMask = MODMASK_QPSK;
			break;
	}

	switch(antAFullRate)
	{
		case 1:
			antAMask = MODMASK_BPSK;
			break;
		case 2:
			antAMask = MODMASK_QPSK;
			break;
		case 4:
			antAMask = MODMASK_16QAM;
			break;
		case 6:
			antAMask = MODMASK_64QAM;
			break;
		case 0xF:
			antAMask = 0xFFFFFFFF;
			break;
		default:
			antAMask = MODMASK_QPSK;
			break;
	}

	switch(antBFullRate)
	{
		case 1:
			antBMask = MODMASK_BPSK;
			break;
		case 2:
			antBMask = MODMASK_QPSK;
			break;
		case 4:
			antBMask = MODMASK_16QAM;
			break;
		case 6:
			antBMask = MODMASK_64QAM;
			break;
		case 0xF:
			antBMask = 0xFFFFFFFF;
			break;
		default:
			antBMask = MODMASK_QPSK;
			break;
	}

	//Configure Tx and Rx antenna A full rate
	for(modIndex=0; modIndex<8; modIndex++)
	{
		*((volatile unsigned int*)(OFDM_BASEADDR+ofdm_txrx_alamouti_SMWO_RxModulation_OFFSET+modIndex*sizeof(int))) =
		(modMasks[modIndex] & antAMask);

		*((volatile unsigned int*)(OFDM_BASEADDR+ofdm_txrx_alamouti_SMWO_TxModulation_OFFSET+modIndex*sizeof(int))) =
			(modMasks[modIndex] & antAMask);
	}


	//Configure Tx and Rx antenna B full rate
	for(modIndex=8; modIndex<16; modIndex++)
	{
		*((volatile unsigned int*)(OFDM_BASEADDR+ofdm_txrx_alamouti_SMWO_RxModulation_OFFSET+modIndex*sizeof(int))) =
		(modMasks[modIndex] & antBMask);

		*((volatile unsigned int*)(OFDM_BASEADDR+ofdm_txrx_alamouti_SMWO_TxModulation_OFFSET+modIndex*sizeof(int))) =
			(modMasks[modIndex] & antBMask);
	}

	//Configure the Tx and Rx base rate
	for(modIndex=16; modIndex<24; modIndex++)
	{
		*((volatile unsigned int*)(OFDM_BASEADDR+ofdm_txrx_alamouti_SMWO_RxModulation_OFFSET+modIndex*sizeof(int))) =
		(modMasks[modIndex] & baseRateMask);

		*((volatile unsigned int*)(OFDM_BASEADDR+ofdm_txrx_alamouti_SMWO_TxModulation_OFFSET+modIndex*sizeof(int))) =
			(modMasks[modIndex] & baseRateMask);
	}
}



void warpmac_setDebugGPIO(unsigned char val){
	XGpio_mSetDataReg(XPAR_DEBUGOUTPUTS_BASEADDR, 1, val);
	return;
}

/*WARPMAC_ENABLEETHERNETINTERRUPT- Allows user's callback to be called.*/
void warpmac_enableEthernetInterrupt(){
	//XIntc_Enable(&myIntc, XPAR_OPB_INTC_0_ETHERNET_MAC_IP2INTC_IRPT_INTR);
	controlStruct.enableEthernetInt=1;
}

/*WARPMAC_DISABLEETHERNETINTERRUPT- Denies user's callback to be called.*/
void warpmac_disableEthernetInterrupt(){
	//-XIntc_Disable(&myIntc, XPAR_OPB_INTC_0_ETHERNET_MAC_IP2INTC_IRPT_INTR);
	controlStruct.enableEthernetInt=0;
}

void warpmac_setBaseRate(unsigned char rate){
	//NOT SUPPORTED IN ALAMOUTI MODEL
	return;
}

void warpmac_setFullRate(unsigned char rate){
	//NOT SUPPORTED IN ALAMOUTI MODEL
	return;
}

void warpmac_setSISOAntenna(unsigned char antSel)
{
	//NOT SUPPORTED IN ALAMOUTI MODEL
	return;
}

void warpmac_multihopCopy(Macframe* packet){




	//if(!((controlStruct.enableSequencing) && (controlStruct.packetDupe))){


	//	memcpy(OFDM_TXRX_MIMO_TXPKTBUFFER_BASEADDR, OFDM_TXRX_MIMO_RXPKTBUFFER_BASEADDR, (size_t)(16 + packet->length + 24));


		memcpy(OFDM_TXRX_ALAMOUTI_TXPKTBUFFER_BASEADDR+6*sizeof(int), OFDM_TXRX_ALAMOUTI_RXPKTBUFFER_BASEADDR+6*sizeof(int), (size_t)(packet->length));



	//}





}



void warpmac_multihopSendOFDM(Macframe* packet){



  int i;
    unsigned int numPayloadBytes;
    unsigned int dataStartIndex;
    unsigned int Npld;
	unsigned char header[24];

	ofdm_pktDetector_mimo_WriteReg_pktDet_reset(PKTDET_BASEADDR, 1);

	//From multiplexing model
	//Npld = ((packet->length + 4 + (6*controlStruct.mod_fullRateA-1))/(6*controlStruct.mod_fullRateA)) * (sisoMode+1);

	Npld = 2 + ((packet->length + 4 + (6*mod_fullRate-1))/(6*mod_fullRate));
	Npld = Npld + (Npld%2);

	numPayloadBytes = 24 + packet->length + 4;

	alamouti_ofdmTx_setNumBytes(numPayloadBytes);
	alamouti_ofdmTx_setNumSyms(numTrainingSyms + (numBaseRate*256) + (Npld*65536));

	header[0]= (packet->destAddr[3]);
	header[1]= (packet->destAddr[2]);
	header[2]= (packet->destAddr[1]);
	header[3]= (packet->destAddr[0]);
	header[4]= (packet->srcAddr[1]);
	header[5]= (packet->srcAddr[0]);
	header[6]= (packet->destAddr[5]);
	header[7]= (packet->destAddr[4]);
	header[8]= (packet->srcAddr[5]);
	header[9]= (packet->srcAddr[4]);
	header[10]= (packet->srcAddr[3]);
	header[11]= (packet->srcAddr[2]);
	header[12]= 0;
	header[13]= packet->pktType;
	header[14]= mod_fullRate;
	header[15]= packet->pktRev;  //version/mode/revision
	header[16]= (controlStruct.mySequenceNum.sequenceNumber[1]); //0
	header[17]= 0;
	header[18]= (0xFF & numPayloadBytes); //less significant byte of total length
	header[19]= (0xFF & (numPayloadBytes >> 8)); //more significant byte of total length
	header[20]= 0;
	header[21]= 0;
	header[22]= 0;
	header[23]= 0;


	alamouti_ofdmTx_copyHeader(header,(size_t)24);
//	mimo_ofdmTx_copyPayload(packet->data, (size_t)(packet->length) );

#ifdef DEBUG_PKT_PRINT
	xil_printf("\r\n******************\r\nTX:");
	xil_printf(" (%d - %d) ",numPayloadBytes, Npld);

	for(i=0;i<24;i++)
	{
		xil_printf("%x.",header[i]);
	}

	xil_printf("\r\n");
#endif

	alamouti_mac2phy_pktTx(TXBLOCK, FIRST_RADIO | SECOND_RADIO);

	WarpRadio_v1_RxEnable(FIRST_RADIO | SECOND_RADIO);

	if(globalReset==0) ofdm_pktDetector_mimo_WriteReg_pktDet_reset(PKTDET_BASEADDR, 0);

	return 0;



}





